//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/DetectorEditor.cpp
//! @brief     Implements class DetectorEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/DetectorEditor.h"
#include "GUI/Model/Detector/DetectorItem.h"
#include "GUI/Model/Detector/ResolutionFunctionItems.h"
#include "GUI/Model/Device/InstrumentItems.h"
#include "GUI/Support/Tool/LayoutUtil.h"
#include "GUI/View/Numeric/ComboUtil.h"
#include "GUI/View/Numeric/DoubleSpinBox.h"
#include "GUI/View/Numeric/NumWidgetUtil.h"
#include "GUI/View/Numeric/SafeSpinBox.h"
#include "GUI/View/Widget/GroupBoxes.h"
#include <QFormLayout>

namespace {

const std::map<FlatDetector::NominalNormal, QString> alignmentDescription = {
    {FlatDetector::X, "Intersection of sample x-axis and detector"},
    {FlatDetector::T, "Intersection of direct beam and detector"},
    {FlatDetector::R, "Intersection of reflected beam and detector"}};

DoubleSpinBox* createSpinBox(QFormLayout* parentFormLayout, DoubleProperty& d,
                             DetectorEditor* parent)
{
    auto* sb = GUI::Util::createDoubleSpinBoxRow(parentFormLayout, d);
    parent->connect(sb, &DoubleSpinBox::baseValueChanged, [parent, &d](double v) {
        if (d.value() != v) {
            d.setValue(v);
            emit parent->dataChanged();
        }
    });
    return sb;
}

} // namespace


DetectorEditor::DetectorEditor(QWidget* parent, GISASInstrumentItem* instrItem)
    : QGroupBox("Detector parameters", parent)
{
    ASSERT(instrItem);

    auto* formLayout = new QFormLayout(this);

    GroupBoxCollapser::installIntoGroupBox2(this, instrItem->expandDetector);

    DetectorItem* detectorItem = instrItem->detectorItem();
    ASSERT(detectorItem);

    auto* bulk = new QVBoxLayout;
    formLayout->addRow(bulk);

    auto* xyrow = new QHBoxLayout;
    bulk->addLayout(xyrow);

    //... x-axis controls

    auto* xAxisFormLayout = new QFormLayout;

    auto* xAxisNbinsSpinBox = new SafeSpinBox;
    xAxisNbinsSpinBox->setRange(1, 65536);
    xAxisNbinsSpinBox->setValue(detectorItem->xSize());
    connect(xAxisNbinsSpinBox, &QSpinBox::valueChanged, [this, detectorItem](int newValue) {
        detectorItem->setXSize(newValue);
        emit dataChanged();
    });
    xAxisFormLayout->addRow("# bins:", xAxisNbinsSpinBox);

    auto* widthSpinBox = new DoubleSpinBox(detectorItem->width());
    connect(widthSpinBox, &DoubleSpinBox::baseValueChanged, [this, detectorItem](double newValue) {
        detectorItem->setWidth(newValue);
        emit dataChanged();
    });
    xAxisFormLayout->addRow("Width (mm):", widthSpinBox);

    ::createSpinBox(xAxisFormLayout, detectorItem->u0(), this);

    auto* xAxisGroupBox = new HeinzGroupBox("X axis", xAxisFormLayout, this);
    xyrow->addWidget(xAxisGroupBox);

    //... y-axis controls

    auto* yAxisFormLayout = new QFormLayout;

    auto* yAxisNbinsSpinBox = new SafeSpinBox;
    yAxisNbinsSpinBox->setRange(1, 65536);
    yAxisNbinsSpinBox->setValue(detectorItem->ySize());
    connect(yAxisNbinsSpinBox, &QSpinBox::valueChanged, [this, detectorItem](int newValue) {
        detectorItem->setYSize(newValue);
        emit dataChanged();
    });
    yAxisFormLayout->addRow("# bins:", yAxisNbinsSpinBox);

    auto* heightSpinBox = new DoubleSpinBox(detectorItem->height());
    connect(heightSpinBox, &DoubleSpinBox::baseValueChanged, [this, detectorItem](double newValue) {
        detectorItem->setHeight(newValue);
        emit dataChanged();
    });
    yAxisFormLayout->addRow("Height (mm):", heightSpinBox);

    ::createSpinBox(yAxisFormLayout, detectorItem->v0(), this);

    auto* yAxisGroupBox = new HeinzGroupBox("Y axis", yAxisFormLayout, this);
    xyrow->addWidget(yAxisGroupBox);

    //... resolution controls

    auto resolutionLayout = new QFormLayout;
    resolutionLayout->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);

    auto* resolutionForm = new HeinzGroupBox("Resolution function", resolutionLayout, this);

    auto updateResolutionForm = [parent = this, layout = resolutionLayout,
                                 item = detectorItem]() -> void {
        while (layout->rowCount() > 1)
            layout->removeRow(1);

        auto* resFunction = item->resolutionFunctionSelection().currentItem();
        if (auto* p = dynamic_cast<ResolutionFunction2DGaussianItem*>(resFunction)) {
            auto* sigmaXSpinBox = GUI::Util::createDoubleSpinBoxRow(layout, p->sigmaX());
            auto* sigmaYSpinBox = GUI::Util::createDoubleSpinBoxRow(layout, p->sigmaY());

            connect(sigmaXSpinBox, &DoubleSpinBox::baseValueChanged, [parent, p](double newValue) {
                p->setSigmaX(newValue);
                emit parent->dataChanged();
            });

            connect(sigmaYSpinBox, &DoubleSpinBox::baseValueChanged, [parent, p](double newValue) {
                p->setSigmaY(newValue);
                emit parent->dataChanged();
            });
        }
    };

    auto* typeCombo = GUI::Util::createComboBoxFromProperty(
        detectorItem->resolutionFunctionSelection(), [this, updateResolutionForm](int) {
            updateResolutionForm();
            emit dataChanged();
        });
    resolutionLayout->addRow("Type:", typeCombo);

    updateResolutionForm();

    xyrow->addWidget(resolutionForm);

    //... detector alignment controls

    auto* aliLine = new QHBoxLayout;
    bulk->addLayout(aliLine);

    QFormLayout* alignmentForm = new QFormLayout;
    alignmentForm->addRow("Alignment:",
                          GUI::Util::createSafeComboBox(
                              [detectorItem] { return detectorItem->detectorAlignmentCombo(); },
                              [this, detectorItem](const QString& t) {
                                  detectorItem->setDetectorAlignment(t);
                                  emit dataChanged();
                              }));
    aliLine->addLayout(alignmentForm);

    auto* detposLayout = new QFormLayout;
    createSpinBox(detposLayout, detectorItem->distance(), this);
    aliLine->addLayout(detposLayout);
}
